@@ -1,5 +1,5 @@
 """
-Copyright 2020 Marco Dal Molin et al.
+Copyright 2019 Marco Dal Molin et al.
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
@@ -29,24 +29,13 @@ import numpy as np
 
 class BaseElement():
     """
-    This is the abstract class for the creation of a BaseElement. A BaseElement
+    This is the abstract class for the cration of a BaseElement. A BaseElement
     does not have parameters or states.
     """
 
     _num_downstream = None
-    """
-    Number of downstream elements
-    """
-
     _num_upstream = None
-    """
-    Number of upstream elements
-    """
-
     input = {}
-    """
-    Dictionary of input fluxes
-    """
 
     def __init__(self, id):
         """
@@ -55,8 +44,8 @@ class BaseElement():
         Parameters
         ----------
         id : str
-            Identifier of the element. All the elements of the framework must
-            have an identifier.
+            Itentifier of the element. All the elements of the framework must
+            have an id.
         """
 
         self.id = id
@@ -65,8 +54,8 @@ class BaseElement():
 
     def set_input(self, input):
         """
-        To be implemented by any child class. It populates the self.input
-        dictionary.
+        To be implemented by any child class. It should transform a list of
+        numpy arrays in a dictionary.
 
         Parameters
         ----------
@@ -84,7 +73,7 @@ class BaseElement():
         Parameters
         ----------
         solve : bool
-            True if the element has to be solved (i.e. calculate the states).
+            True if the element has to be solved (i.e. calcualte the states).
 
         Returns
         -------
@@ -126,14 +115,11 @@ class BaseElement():
 
 class ParameterizedElement(BaseElement):
     """
-    This is the abstract class for the creation of a ParameterizedElement. A
+    This is the abstract class for the cration of a ParameterizedElement. A
     ParameterizedElement has parameters but not states.
     """
 
     _prefix_parameters = ''
-    """
-    Prefix applied to the original names of the parameters
-    """
 
     def __init__(self, parameters, id):
         """
@@ -146,8 +132,8 @@ class ParameterizedElement(BaseElement):
             a float (constant in time) or a numpy.ndarray of the same length
             of the input fluxes (time variant parameters).
         id : str
-            Identifier of the element. All the elements of the framework must
-            have an identifier.
+            Itentifier of the element. All the elements of the framework must
+            have an id.
         """
 
         BaseElement.__init__(self, id)
@@ -163,7 +149,7 @@ class ParameterizedElement(BaseElement):
         ----------
         names : list(str)
             Names of the parameters to return. The names must be the ones
-            returned by the method get_parameters_name. If None, all the
+            returned by the mehod get_parameters_name. If None, all the
             parameters are returned.
 
         Returns
@@ -259,14 +245,11 @@ class ParameterizedElement(BaseElement):
 
 class StateElement(BaseElement):
     """
-    This is the abstract class for the creation of a StateElement. A
+    This is the abstract class for the cration of a StateElement. A
     StateElement has states but not parameters.
     """
 
     _prefix_states = ''
-    """
-    Prefix applied to the original names of the parameters
-    """
 
     def __init__(self, states, id):
         """
@@ -278,7 +261,7 @@ class StateElement(BaseElement):
             Initial states of the element. Depending on the element the states
             can be either a float or a numpy.ndarray.
         id : str
-            Identifier of the element. All the elements of the framework must
+            Itentifier of the element. All the elements of the framework must
             have an id.
         """
         BaseElement.__init__(self, id)
@@ -295,7 +278,7 @@ class StateElement(BaseElement):
         ----------
         names : list(str)
             Names of the states to return. The names must be the ones
-            returned by the method get_states_name. If None, all the
+            returned by the mehod get_states_name. If None, all the
             states are returned.
 
         Returns
@@ -402,7 +385,7 @@ class StateElement(BaseElement):
 
 class StateParameterizedElement(StateElement, ParameterizedElement):
     """
-    This is the abstract class for the creation of a StateParameterizedElement.
+    This is the abstract class for the cration of a StateParameterizedElement.
     A StateParameterizedElement has parameters and states.
     """
 
@@ -421,7 +404,7 @@ class StateParameterizedElement(StateElement, ParameterizedElement):
             Initial states of the element. Depending on the element the states
             can be either a float or a numpy.ndarray.
         id : str
-            Identifier of the element. All the elements of the framework must
+            Itentifier of the element. All the elements of the framework must
             have an id.
         """
 
@@ -463,47 +446,17 @@ class StateParameterizedElement(StateElement, ParameterizedElement):
 
 class ODEsElement(StateParameterizedElement):
     """
-    This is the abstract class for the creation of a ODEsElement. An ODEsElement
+    This is the abstract class for the cration of a ODEsElement. An ODEsElement
     is an element with states and parameters that is controlled by an ordinary
     differential equation, of the form:
 
     dS/dt = input - output
     """
-
     _num_upstream = 1
-    """
-    Number of upstream elements
-    """
-
     _num_downstream = 1
-    """
-    Number of downstream elements
-    """
-
     _solver_states = []
-    """
-    List of states used by the solver of the differential equation
-    """
 
-    _fluxes = []
-    """
-    This attribute contains a list of methods (one per differential equation)
-    that calculate the values of the fluxes needed to solve the differential
-    equations that control the element. The single functions must return the
-    fluxes as a list where incoming fluxes are positive and outgoing are
-    negative. Here is a list of the required outputs of the single functions:
-
-    list(floats)
-        Values of the fluxes given states, inputs, and parameters.
-    float
-        Minimum value of the state. Used, sometimes, by the numerical solver
-        to search for the solution.
-    float
-        Maximum value of the state. Used, sometimes, by the numerical solver
-        to search for the solution.
-    """
-
-    def __init__(self, parameters, states, approximation, id):
+    def __init__(self, parameters, states, solver, id):
         """
         This is the initializer of the abstract class ODEsElement.
 
@@ -516,17 +469,19 @@ class ODEsElement(StateParameterizedElement):
         states : dict
             Initial states of the element. Depending on the element the states
             can be either a float or a numpy.ndarray.
-        approximation : superflexpy.utils.numerical_approximation.NumericalApproximator
-            Numerial method used to approximate the differential equation
+        solver : superflexpy.utils.root_finder.RootFinder
+            Solver used to find the root(s) of the differential equation(s).
+            Child classes may implement their own solver, therefore the tipe
+            of the solver is not enforced.
         id : str
-            Identifier of the element. All the elements of the framework must
+            Itentifier of the element. All the elements of the framework must
             have an id.
         """
 
         StateParameterizedElement.__init__(self, parameters=parameters,
                                            states=states, id=id)
 
-        self._num_app = approximation
+        self._solver = solver
 
     def set_timestep(self, dt):
         """
@@ -550,7 +505,7 @@ class ODEsElement(StateParameterizedElement):
         """
         return self._dt
 
-    def define_numerical_approximation(self, approximation):
+    def define_solver(self, solver):
         """
         This method define the solver to use for the differential equation.
 
@@ -558,29 +513,32 @@ class ODEsElement(StateParameterizedElement):
         ----------
         solver : superflexpy.utils.root_finder.RootFinder
             Solver used to find the root(s) of the differential equation(s).
-            Child classes may implement their own solver, therefore the type
+            Child classes may implement their own solver, therefore the tipe
             of the solver is not enforced.
         """
 
-        self._num_app = approximation
+        self._solver = solver
 
     def _solve_differential_equation(self, **kwargs):
-        """
-        This method calls the solver of the differential equation(s). When
-        called, it solves the differential equation(s) for all the timesteps
-        and populates self.state_array.
-        """
-
         if len(self._solver_states) == 0:
             message = '{}the attribute _solver_states must be filled'.format(self._error_message)
             raise ValueError(message)
 
-        self.state_array = self._num_app.solve(fun=self._fluxes,
-                                               S0=self._solver_states,
-                                               dt=self._dt,
-                                               **self.input,
-                                               **{k[len(self._prefix_parameters):]: self._parameters[k] for k in self._parameters},
-                                               **kwargs)
+        self.state_array = self._solver.solve(fun=self._differential_equation,
+                                              S0=self._solver_states,
+                                              dt=self._dt,
+                                              **self.input,
+                                              **{k[len(self._prefix_parameters):]: self._parameters[k] for k in self._parameters},
+                                              **kwargs)
+
+    def _differential_equation(self):
+        """
+        To be implemented by any child class. This method sets the differential
+        equation(s) to be solved by the solver. The method must be implemented
+        in order to satisfy the requirements of the solver.
+        """
+
+        raise NotImplementedError('The _differential_equation method must be implemented')
 
     def __copy__(self):
         p = self._parameters  # Only the reference
@@ -588,7 +546,7 @@ class ODEsElement(StateParameterizedElement):
         ele = self.__class__(parameters=p,
                              states=s,
                              id=self.id,
-                             approximation=self._num_app)
+                             solver=self._solver)
         ele._prefix_states = self._prefix_states
         ele._prefix_parameters = self._prefix_parameters
         return ele
@@ -599,7 +557,7 @@ class ODEsElement(StateParameterizedElement):
         ele = self.__class__(parameters=p,
                              states=s,
                              id=self.id,
-                             approximation=self._num_app)
+                             solver=self._solver)
         ele._prefix_states = self._prefix_states
         ele._prefix_parameters = self._prefix_parameters
         return ele
@@ -607,43 +565,30 @@ class ODEsElement(StateParameterizedElement):
 
 class LagElement(StateParameterizedElement):
     """
-    This is the abstract class for the creation of a LagElement. An LagElement
+    This is the abstract class for the cration of a LagElement. An LagElement
     is an element with states and parameters that distributes the incoming
     fluxes according to a weight array
 
     Parameters must be called:
-
     - 'lag-time': characteristic time of the lag. Its definition depends on the
-      specific implementations of the element. It can be a scalar (it will be
-      applied to all the fluxes) or a list (with length equal to the number of
-      fluxes).
+                  specific implementations of the element. It can be a scalar
+                  (it will be applied to all the fluxes) of a list (with length
+                  equal to the number of fluxes).
 
     States must be called:
-
     - lag: initial state of the lag function. If None it will be initialized
-      to zeros. It can be a numpy.ndarray (it will be applied to all the fluxes)
-      of a list on numpy.ndarray (with length equal to the number of fluxes).
+           to zeros. It can be a numpy.ndarray (it will be applied to all the
+           fluxes) of a list on numpy.ndarray (with length equal to the number
+           of fluxes).
     """
 
     _num_upstream = 1
-    """
-    Number of upstream elements
-    """
-
     _num_downstream = 1
-    """
-    Number of downstream elements
-    """
 
     def _build_weight(self, lag_time):
         """
         This method must be implemented by any child class. It calculates the
-        weight array(s) based on the lag_time.
-
-        Parameters
-        ----------
-        lag_time : float
-            Characteristic time of the lag function.
+        weight array(s) based on th lag_time.
 
         Returns
         -------
@@ -659,7 +604,6 @@ class LagElement(StateParameterizedElement):
         inputs is not important, the fluxes are stored as list.
 
         Parameters
-        ----------
         input : list(numpy.ndarray)
             List of input fluxes.
         """
@@ -674,7 +618,7 @@ class LagElement(StateParameterizedElement):
         Parameters
         ----------
         solve : bool
-            True if the element has to be solved (i.e. calculate the states).
+            True if the element has to be solved (i.e. calcualte the states).
 
         Returns
         -------
@@ -719,38 +663,8 @@ class LagElement(StateParameterizedElement):
 
         return [self.state_array[:, i, 0] for i in range(len(self.input))]
 
-    def reset_states(self):
-        """
-        This method sets the states to the values provided to the __init__
-        method. In this case, if a state was initialized as None, it will be
-        set back to None.
-        """
-
-        for k in self._init_states.keys():
-            k_no_prefix = k.split('_')[-1]
-            self._states[self._prefix_states + k_no_prefix] = deepcopy(self._init_states[k])  # I have to isolate
-
     @staticmethod
     def _solve_lag(weight, lag_state, input):
-        """
-        This method distributes the input fluxes according to the weight array
-        and the initial state.
-
-        Parameters
-        ----------
-        weight : list(numpy.ndarray)
-            List of weights to use
-        lag_state : list(numpy.ndarray)
-            List of the initial states of the lag.
-        input : list(numpy.ndarray)
-            List of fluxes
-
-        Returns
-        -------
-        numpy.ndarray
-            3D array (dimensions: number of timesteps, number of fluxes, max
-            lag length) that stores all the states of the lag in time
-        """
 
         max_length = max([len(w) for w in weight])
 
@@ -765,20 +679,6 @@ class LagElement(StateParameterizedElement):
         return output
 
     def _init_lag_state(self, lag_time):
-        """
-        This method sets the initial state of the lag to arrays of proper
-        length.
-
-        Parameters
-        ----------
-        lag_time : list(float)
-            List of lag times
-
-        Returns
-        -------
-        list(numpy.ndarray)
-            List of the initial states of the lag.
-        """
 
         ini_state = []
         for i in range(len(self.input)):
