@@ -1,5 +1,5 @@
 """
-Copyright 2020 Marco Dal Molin et al.
+Copyright 2019 Marco Dal Molin et al.
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
@@ -22,11 +22,10 @@ DESIGNED BY: Marco Dal Molin, Fabrizio Fenicia
 This file contains the implementation of the Network class.
 """
 
-from ..utils.generic_component import GenericComponent
-from .node import Node
+from ..utils.common_class import CommonClass
 
 
-class Network(GenericComponent):
+class Network(CommonClass):
     """
     This class defines a Network. A network is a collection of Nodes and it is
     used to route the fluxes from upstream to downstream. A network must be a
@@ -48,17 +47,12 @@ class Network(GenericComponent):
             be a tree, each key has only one downstream element
         """
 
-        self._error_message = 'module : superflexPy, Network ,'
-        self._error_message += ' Error message : '
-
-        for n in nodes:
-            if not isinstance(n, Node):
-                message = '{}units must be instance of the Unit class'.format(self._error_message)
-                raise TypeError(message)
-
         self._content = nodes
         self._downstream = topography
 
+        self._error_message = 'module : superflexPy, Network ,'
+        self._error_message += ' Error message : '
+
         self._build_network()
 
     # METHODS FOR THE USER
@@ -71,11 +65,11 @@ class Network(GenericComponent):
         Parameters
         ----------
         solve : bool
-            True if the elements have to be solved (i.e. calculate the states).
+            True if the elements have to be solved (i.e. calcualte the states).
 
         Returns
         -------
-        :dict(str : list(numpy.ndarray))
+        dict(str : list(numpy.ndarray))
             Dictionary containig the output fluxes of all the nodes.
         """
 
@@ -145,19 +139,13 @@ class Network(GenericComponent):
             Name of the attribute to expose.
         Returns
         -------
-        Unknown
-            Attribute exposed
+        Attribute exposed
         """
 
         cat_num, ele = self._find_attribute_from_name(id)
 
         if ele:
-            splitted = id.split('_')
-            if len(splitted) == 3:  # element
-                ele_id = splitted[1] + '_' + splitted[2]
-            else:  # unit
-                ele_id = splitted[1]
-            return self._content[cat_num].get_internal(ele_id, attribute)
+            return self._content[cat_num].get_internal(id, attribute)
         else:
             try:
                 method = getattr(self._content[cat_num], attribute)
@@ -182,19 +170,13 @@ class Network(GenericComponent):
 
         Returns
         -------
-        Unknown
-            Output of the called method.
+        Output of the called method.
         """
 
         cat_num, ele = self._find_attribute_from_name(id)
 
         if ele:
-            splitted = id.split('_')
-            if len(splitted) == 3:  # element
-                ele_id = splitted[1] + '_' + splitted[2]
-            else:  # unit
-                ele_id = splitted[1]
-            return self._content[cat_num].call_internal(ele_id, method, **kwargs)
+            return self._content[cat_num].call_internal(id, method, **kwargs)
         else:
             try:
                 method = getattr(self._content[cat_num], method)
@@ -206,9 +188,6 @@ class Network(GenericComponent):
     # PROTECTED METHODS
 
     def _build_network(self):
-        """
-        This method constructs all the structures needed to solve the network
-        """
 
         # Find the upstream catchments
         self._upstream = {k: [] for k in self._downstream.keys()}
@@ -262,25 +241,10 @@ class Network(GenericComponent):
                             completed = True
 
     def _find_attribute_from_name(self, id):
-        """
-        This method is used to find the attributes or methods of the components
-        contained for post-run inspection.
-
-        Parameters
-        ----------
-        id : str
-            Identifier of the component
-
-        Returns
-        -------
-        int, bool
-            Index of the component to look for and indication if it is an
-            element or a unit (True) or not.
-        """
 
         splitted = id.split('_')
 
-        cat_num = self._find_content_from_name(id)  # Options: None if cat_id not in id, number otherwise
+        cat_num = self._find_content_from_name(id)
 
         if len(splitted) >= 2:
             return (cat_num, True)  # We are looking for a HRU or an element
