@@ -1,5 +1,5 @@
 """
-Copyright 2020 Marco Dal Molin et al.
+Copyright 2019 Marco Dal Molin et al.
 
 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
@@ -23,18 +23,16 @@ This file contains the implementation of the Node class.
 """
 
 from copy import copy, deepcopy
-from ..utils.generic_component import GenericComponent
-from .unit import Unit
+from ..utils.common_class import CommonClass
 
 
-class Node(GenericComponent):
+class Node(CommonClass):
     """
     This class defines a Node. A node can be part of a network and it is a
     collection of Units. It's task is to sum the outputs of the Units,
     applying, if present, a routing.
     """
-
-    def __init__(self, units, weights, area, id, parameters=None, states=None, shared_parameters=True):
+    def __init__(self, units, weights, area, id, shared_parameters=True):
         """
         This is the initializer of the class Node.
 
@@ -51,8 +49,8 @@ class Node(GenericComponent):
             Influence area of the node. It is the net value: if a node has
             other nodes upstream, their area is not counted.
         id : str
-            Identifier of the node. All the nodes of the framework must have an
-            identifier.
+            Itentifier of the node. All the nodes of the framework must have an
+            id.
         shared_parameters : bool
             True if the parameters of the Units are shared among the different
             Nodes.
@@ -63,29 +61,18 @@ class Node(GenericComponent):
         self._error_message = 'module : superflexPy, Node : {},'.format(id)
         self._error_message += ' Error message : '
 
-        # Handle local parameters and states
-        if parameters is not None:
-            self._local_parameters = parameters
-
-        if states is not None:
-            self._local_states = states
-            self._init_local_states = deepcopy(states)
-
         self._content = []
         for h in units:
-            if not isinstance(h, Unit):
-                message = '{}units must be instance of the Unit class'.format(self._error_message)
-                raise TypeError(message)
             if shared_parameters:
                 self._content.append(copy(h))
             else:
                 self._content.append(deepcopy(h))
+                self.add_prefix_parameters(id)
 
         self.area = area
         self._content_pointer = {hru.id: i
                                  for i, hru in enumerate(self._content)}
         self._weights = deepcopy(weights)
-        self.add_prefix_parameters(id, shared_parameters)
         self.add_prefix_states(id)
 
     # METHODS FOR THE USER
@@ -110,7 +97,7 @@ class Node(GenericComponent):
         Parameters
         ----------
         solve : bool
-            True if the elements have to be solved (i.e. calculate the states).
+            True if the elements have to be solved (i.e. calcualte the states).
 
         Returns
         -------
@@ -171,15 +158,13 @@ class Node(GenericComponent):
 
         Returns
         -------
-        Unknown
-            Attribute exposed
+        Attribute exposed
         """
 
         hru_num, ele = self._find_attribute_from_name(id)
 
         if ele:
-            ele_id = id.split('_')[-1]
-            return self._content[hru_num].get_internal(ele_id, attribute)
+            return self._content[hru_num].get_internal(id, attribute)
         else:
             try:
                 method = getattr(self._content[hru_num], attribute)
@@ -204,14 +189,12 @@ class Node(GenericComponent):
 
         Returns
         -------
-        Unknown
-            Output of the called method.
+        Output of the called method.
         """
         hru_num, ele = self._find_attribute_from_name(id)
 
         if ele:
-            ele_id = id.split('_')[-1]
-            return self._content[hru_num].call_internal(ele_id, method, **kwargs)
+            return self._content[hru_num].call_internal(id, method, **kwargs)
         else:
             try:
                 method = getattr(self._content[hru_num], method)
@@ -222,7 +205,7 @@ class Node(GenericComponent):
 
     # METHODS USED BY THE FRAMEWORK
 
-    def add_prefix_parameters(self, id, shared_parameters):
+    def add_prefix_parameters(self, id):
         """
         This method adds the prefix to the parameters of the elements that are
         contained in the node.
@@ -233,27 +216,8 @@ class Node(GenericComponent):
             Prefix to add.
         """
 
-        # Add prefix to local parameters
-        if '_' in id:
-            message = '{}The prefix cannot contain \'_\''.format(self._error_message)
-            raise ValueError(message)
-
-        if self._local_parameters:  # the following block runs only if the dictionary is not empty
-            # Extract the prefixes in the parameters name
-            splitted = list(self._local_parameters.keys())[0].split('_')
-
-            if id not in splitted:
-                # Apply the prefix
-                for k in list(self._local_parameters.keys()):
-                    value = self._local_parameters.pop(k)
-                    self._local_parameters['{}_{}'.format(id, k)] = value
-
-                # Save the prefix for furure uses
-                self._prefix_local_parameters = '{}_{}'.format(id, self._prefix_local_parameters)
-
-        if not shared_parameters:
-            for h in self._content:
-                h.add_prefix_parameters(id)
+        for h in self._content:
+            h.add_prefix_parameters(id)
 
     def add_prefix_states(self, id):
         """
@@ -266,26 +230,6 @@ class Node(GenericComponent):
             Prefix to add.
         """
 
-        # Add prefix to local states
-        if '_' in id:
-            message = '{}The prefix cannot contain \'_\''.format(self._error_message)
-            raise ValueError(message)
-
-        if self._local_states:  # the following block runs only if the dictionary is not empty
-            # Extract the prefixes in the parameters name
-            splitted = list(self._local_states.keys())[0].split('_')
-
-            if id not in splitted:
-                # Apply the prefix
-                for k in list(self._local_states.keys()):
-                    value = self._local_states.pop(k)
-                    self._local_states['{}_{}'.format(id, k)] = value
-                    value = self._init_local_states.pop(k)
-                    self._init_local_states['{}_{}'.format(id, k)] = value
-
-                # Save the prefix for furure uses
-                self._prefix_local_states = '{}_{}'.format(id, self._prefix_local_states)
-
         for h in self._content:
             h.add_prefix_states(id)
 
@@ -308,38 +252,21 @@ class Node(GenericComponent):
     # PROTECTED METHODS
 
     def _find_attribute_from_name(self, id):
-        """
-        This method is used to find the attributes or methods of the components
-        contained for post-run inspection.
-
-        Parameters
-        ----------
-        id : str
-            Identifier of the component
-
-        Returns
-        -------
-        int, bool
-            Index of the component to look for and indication if it is an
-            element (True) or not.
-        """
 
         splitted = id.split('_')
 
         hru_num = self._find_content_from_name(id)
 
-        if len(splitted) == 2:
+        if len(splitted) == 3:
             return (hru_num, True)  # We are looking for an element
-        elif len(splitted) == 1:
-            return (hru_num, False)  # We are looking for a unit
         else:
-            raise ValueError('Tmp for debug in node')
+            return (hru_num, False)
 
     def _internal_routing(self, flux):
         """
-        Internal routing is the one that affects the flux coming to the Units
-        and reaching the outflow of the node. This function is internally
-        used by the node.
+        Internal routing is the one that affects the flux coming to the HRUs
+        and reaching the outflow of the catchment. This function is internally
+        used by the Catchment.
         """
 
         # No routing
